<?php

namespace App\Jobs;

use App\Models\TpFlight;
use App\Models\TpProduct;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;

class TravelportReferenceListSavingJob implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    protected string $catalogProductOfferingId;
    protected Collection $referenceList;

    /**
     * Create a new job instance.
     */
    public function __construct(string $catalogProductOfferingId, Collection $referenceList)
    {
        $this->catalogProductOfferingId = $catalogProductOfferingId;
        $this->referenceList = $referenceList;
    }

    /**
     * Execute the job.
     */
    public function handle(): void
    {
        Log::info("Queue started");
        $this->referenceList->map(function ($content) {
            $contentSource = $content['source'];
            $catalogId = $content['catalogId'];

            $products = collect($content['content']->where("@type", "ReferenceListProduct")->first()["Product"])
                ->map(fn($product) => [
                    'catalogIdentifier' => $this->catalogProductOfferingId,
                    'productId' => $product['id'],
                    'source' => $contentSource,
                    'flightSegment' => json_encode($product['FlightSegment']),
                    'passengerFlight' => json_encode($product['PassengerFlight']),
                    'totalDuration' => $product['totalDuration'],
                ]);

            TpProduct::insert($products->toArray());
            Log::info("Products Inserted");

            $flights = collect($content['content']->where("@type", "ReferenceListFlight")->first()["Flight"])
                ->map(fn($flight) => [
                    'catalogIdentifier' => $this->catalogProductOfferingId,
                    'flightId' => $flight['id'],
                    'source' => $contentSource,
                    'carrier' => $flight['carrier'],
                    'equipment' => $flight['equipment'],
                    'number' => $flight['number'],
                    'duration' => $flight['duration'] ?? '',
                    'distance' => $flight['distance'] ?? '',
                    'stops' => $flight['stops'] ?? '',
                    'arrival' => json_encode($flight['Arrival']),
                    'departure' => json_encode($flight['Departure']),
                ]);

            TpFlight::insert($flights->toArray());
            Log::info("Flights Inserted");
        });
    }
}
